
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function Control(overlay)
{
	this.overlay     = overlay;
	this.prefBranch  = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	this.prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

	this.BOOLEAN = "boolean";
	this.INTEGER = "integer";
	this.STRING  = "string";

	this.UPDATE_ALL      = "updateAll";
	this.UPDATE_ONE      = "updateOne";
	this.UPDATE_PROFILES = "updateProfiles";
	this.LOAD_PROFILE    = "loadProfile";

	this.ROOT    = "extensions.prefswitch.";
	this.REFRESH = "refresh";
	this.BRANCH  = this.prefService.getBranch(this.ROOT);

	this.SWITCH_PATH  = "extensions/prefswitch@mozdev.org/switch.xml";
	this.PROFILE_PATH = "extensions/prefswitch@mozdev.org/profile.xml";

// -----------------------------------------------------------------------------

	this.getValue = function(preference, defaultValue, type)
	{
		try
		{
			switch (type)
			{
				case this.BOOLEAN: return this.prefBranch.getBoolPref(preference);
				case this.INTEGER: return this.prefBranch.getIntPref(preference);
				case this.STRING:  return this.prefBranch.getCharPref(preference);
			}
		}
		catch (e)
		{
			if (defaultValue == null) return null;
			this.setValue(preference, defaultValue, type);
		}
		return this.getValue(preference, null, type);
	};

	this.setValue = function(preference, value, type)
	{
		switch (type)
		{
			case this.BOOLEAN:
				this.prefBranch.setBoolPref(preference, value);
				break;
			case this.INTEGER:
				this.prefBranch.setIntPref(preference, value);
				break;
			case this.STRING:
				this.prefBranch.setCharPref(preference, value);
		}
	};

	this.update = function(type, statusbarpanel)
	{
		if (this.overlay)
		{
			switch (type)
			{
				case this.UPDATE_ALL:
					this.overlay.updateAll();
					break;
				case this.UPDATE_ONE:
					this.overlay.updateOne(statusbarpanel);
					break;
				case this.UPDATE_PROFILES:
					this.overlay.updateProfiles();
			}
		}
		else
		{
			var value = !this.BRANCH.getBoolPref(this.REFRESH);
			this.BRANCH.setBoolPref(this.REFRESH, value);
		}
	};

// -----------------------------------------------------------------------------

	this.getSwitches = function()
	{
		var aResult = [];
		var doc = this.getDoc(this.SWITCH_PATH);
		if (doc.documentElement.nodeName == "parsererror")
		{
			dump("Control.getSwitches(): Error while parsing.\n");
			return [];
		}
		var iSwitch = document.evaluate("//switch", doc, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
		var nSwitch = iSwitch.iterateNext();
		while (nSwitch)
		{
			var tSwitch = new Switch(
				nSwitch.getAttribute("name"),
				nSwitch.getAttribute("show"),
				nSwitch.getAttribute("preference"),
				nSwitch.getAttribute("initial"),
				nSwitch.getAttribute("type"),
				nSwitch.getAttribute("tooltipPrefix"),
				nSwitch.getAttribute("states"));
			var iState = document.evaluate(".//state", nSwitch, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nState = iState.iterateNext();
			while (nState)
			{
				tSwitch.addState(
					nState.getAttribute("value"),
					nState.getAttribute("description"),
					nState.getAttribute("tooltipPostfix"),
					nState.getAttribute("color"));
				nState = iState.iterateNext();
			}
			aResult.push(tSwitch);
			nSwitch = iSwitch.iterateNext();
		}
		return aResult;
	};

	this.saveSwitches = function(data)
	{
		this.saveToFile(data, this.SWITCH_PATH);
	};

// -----------------------------------------------------------------------------

	this.getProfiles = function()
	{
		var aResult = [];
		var doc = this.getDoc(this.PROFILE_PATH);
		if (doc.documentElement.nodeName == "parsererror")
		{
			dump("Control.getProfiles(): Error while parsing.\n");
			return [];
		}
		var iProfile = document.evaluate("//profile", doc, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
		var nProfile = iProfile.iterateNext();
		while (nProfile)
		{
			var tProfile = new Profile(nProfile.getAttribute("name"));
			var iFeature = document.evaluate(".//feature", nProfile, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
			var nFeature = iFeature.iterateNext();
			while (nFeature)
			{
				tProfile.addFeature(
					nFeature.getAttribute("key"),
					nFeature.getAttribute("value"));
				nFeature = iFeature.iterateNext();
			}
			aResult.push(tProfile);
			nProfile = iProfile.iterateNext();
		}
		return aResult;	
	};

	this.saveProfiles = function(data)
	{
		this.saveToFile(data, this.PROFILE_PATH);
	};

// -----------------------------------------------------------------------------

	this.getFile = function(path)
	{
		var file  = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		var aPath = path.split(/\/|\\/);
		for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
		return file;
	};

	this.getDoc = function(path)
	{
		var fiStream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
		var siStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
		var file     = this.getFile(path);
		fiStream.init(file, 1, 0, false);
		siStream.init(fiStream);
		var data = siStream.read(-1);
		siStream.close();
		fiStream.close();
		var parser = new DOMParser();
		return parser.parseFromString(data, "text/xml");
	};

	this.saveToFile = function(data, path)
	{
		var parser     = new DOMParser();
		var doc        = parser.parseFromString(data, "text/xml");
		var file       = this.getFile(path);
		var foStream   = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
		var serializer = new XMLSerializer();
		serializer.serializeToStream(doc, foStream, "");
		foStream.close();
	};
}
