
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const BOOLEAN = 0;
const INTEGER = 1;
const STRING  = 2;

const BLACK     = 0;
const DARKGREY  = 1;
const LIGHTGREY = 2;

function SwitchDialog()
{
	this.control        = null;
	this.listbox        = null;
	this.dialogheader   = null;
	this.show           = null;
	this.preference     = null;
	this.initial        = null;
	this.type           = null;
	this.tooltipPrefix  = null;
	this.value          = null;
	this.description    = null;
	this.tooltipPostfix = null;
	this.color          = null;
	this.tree           = null;
	this.columns        =
	{
		value:          [0, 0], // colName: [index, order]
		description:    [1, 0],
		tooltipPostfix: [2, 0],
		color:          [3, 0]
	};
	this.add     = null;
	this.update  = null;
	this.tSwitch = null;
	this.index   = null;
	this.file    = null;

// -----------------------------------------------------------------------------

	this.setShow          = function(value) { this.tSwitch.show          = "" + value; };
	this.setPreference    = function(value) { this.tSwitch.preference    = value; };
	this.setInitial       = function(value) { this.tSwitch.initial       = value; };
	this.setType          = function(value) { this.tSwitch.type          = value; };
	this.setTooltipPrefix = function(value) { this.tSwitch.tooltipPrefix = value; };

// -----------------------------------------------------------------------------

	this.addState = function()
	{
		if (this.value.value == "") return;
		this.tSwitch.addState(this.value.value, this.description.value, this.tooltipPostfix.value, this.color.selectedItem.label);
		this.refreshSwitches();
	};

	this.updateState = function()
	{
		if (this.value.value == "") return;
		this.tSwitch.states[this.index][0] = this.value.value;
		this.tSwitch.states[this.index][1] = this.description.value;
		this.tSwitch.states[this.index][2] = this.tooltipPostfix.value;
		this.tSwitch.states[this.index][3] = this.color.selectedItem.label;
		this.refreshSwitches();
	};

	this.removeState = function()
	{
		if (this.index >= 0) this.tSwitch.removeState(this.index);
		this.refreshSwitches();
	};

	this.removeAllStates = function()
	{
		this.tSwitch.states = [];
		this.refreshSwitches();
	};

	this.newSwitch = function()
	{
		this.tSwitch  = new Switch("New Switch", "true", "", "", "boolean", "");
		var listitem = this.listbox.appendItem(this.tSwitch.name);
		listitem.sw  = this.tSwitch;
		this.listbox.selectedIndex = this.listbox.getRowCount() - 1;
		this.refreshSwitches();
		if (!this.renameSwitch())
		{
			this.listbox.removeItemAt(this.listbox.selectedIndex);
			if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
			else this.refreshSwitches();
		}
	};

	this.renameSwitch = function()
	{
		if (this.tSwitch == null) return false;
		var name = prompt("Enter name of switch:", this.tSwitch.name);
		if (name == null || name == "") return false;
		this.tSwitch.name = name;
		this.refreshSwitches();
		return true;
	};

	this.removeSwitch = function()
	{
		if (this.listbox.selectedIndex < 0) return;
		this.listbox.removeItemAt(this.listbox.selectedIndex);
		if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
	};

// -----------------------------------------------------------------------------

	this.load = function()
	{
		if("arguments" in window && window.arguments.length > 0) this.control = window.arguments[0];
		else this.control = new Control();
		this.listbox        = document.getElementById("listbox");
		this.dialogheader   = document.getElementById("dialogheader");
		this.show           = document.getElementById("show");
		this.preference     = document.getElementById("preference");
		this.initial        = document.getElementById("initial");
		this.type           = document.getElementById("type");
		this.tooltipPrefix  = document.getElementById("tooltipPrefix");
		this.value          = document.getElementById("valueTB");
		this.description    = document.getElementById("descriptionTB");
		this.tooltipPostfix = document.getElementById("tooltipPostfixTB");
		this.color          = document.getElementById("colorML");
		this.tree           = document.getElementById("tree");
		this.add            = document.getElementById("add");
		this.update         = document.getElementById("update");
		this.listbox.addEventListener("select", this, false);
		this.tree.addEventListener("select", this, false);
		this.loadSwitches();
	};

	this.handleEvent = function(event)
	{
		switch (event.target.tagName)
		{
			case "listbox":
				this.refreshSwitches();
				break;
			case "tree":
				this.refreshStates();
		}
	};

	this.loadSwitches = function()
	{
		var aSwitch = this.control.getSwitches();
		for (var i = 0; i < aSwitch.length; i++)
		{
			var tSwitch  = aSwitch[i];
			var listitem = this.listbox.appendItem(tSwitch.name);
			listitem.sw  = tSwitch;
		}
		if (this.listbox.getRowCount() > 0 && this.listbox.selectedIndex < 0) this.listbox.selectedIndex = 0;
	};

	this.refreshSwitches = function()
	{
		if (this.listbox.selectedIndex < 0)
		{
			this.dialogheader.setAttribute("title", "");
			this.show.selected        = false;
			this.preference.value     = "";
			this.initial.value        = "";
			this.type.selectedIndex   = 0;
			this.tooltipPrefix.value  = "";
			this.value.value          = "";
			this.description.value    = "";
			this.tooltipPostfix.value = "";
			this.color.selectedIndex  = 0;
			this.tree.view            = new TreeView([], this.columns, 0);
		}
		else
		{
			var tListitem = this.listbox.getSelectedItem(0);
			if (tListitem == null) return;
			this.tSwitch = tListitem.sw;
			if (this.tSwitch == null) return;
			tListitem.label = this.tSwitch.name;
			this.dialogheader.setAttribute("title", this.tSwitch.name);
			this.show.checked      = this.tSwitch.show == "true";
			this.preference.value  = this.tSwitch.preference;
			this.initial.value     = this.tSwitch.initial;
			switch (this.tSwitch.type)
			{
				case "integer": this.type.selectedIndex = INTEGER; break;
				case "string":  this.type.selectedIndex = STRING; break;
				default:        this.type.selectedIndex = BOOLEAN;
			}
			this.tooltipPrefix.value = this.tSwitch.tooltipPrefix;
			this.tree.view           = new TreeView(this.tSwitch.states, this.columns, this.tSwitch.states.length);
			this.refreshStates();
		}
	};

	this.columns        =
	{
		value:          [0, 0], // colName: [index, order]
		description:    [1, 0],
		tooltipPostfix: [2, 0],
		color:          [3, 0]
	};

	this.sortColumn = function(column)
	{
		var order = this.columns[column.id][1];
		this.columns[column.id][1] = order == 0 ? 1 : 0;
		this.tableSort(this.tSwitch.states, this.columns[column.id][0], order);
		this.tree.view = new TreeView(this.tSwitch.states, this.columns, this.tSwitch.states.length);
	};

	this.tableSort = function(table, col, order)
	{
		var columnSort;
		if (order == 0) columnSort = function(a, b) { return a[col] < b[col] ? -1 : a[col] > b[col] ? 1 : 0; } // ascending
		else columnSort = function(a, b) { return a[col] > b[col] ? -1 : a[col] < b[col] ? 1 : 0; }            // descending
		table.sort(columnSort);
	};

	this.refreshStates = function()
	{
		this.index = this.tree.currentIndex;
		if (this.index < 0)
		{
			this.update.hidden        = true;
			this.value.value          = "";
			this.description.value    = "";
			this.tooltipPostfix.value = "";
			this.color.selectedIndex  = 0;
			return;
		}
		else
		{
			this.update.hidden        = false;
			this.value.value          = this.tSwitch.states[this.index][0];
			this.description.value    = this.tSwitch.states[this.index][1];
			this.tooltipPostfix.value = this.tSwitch.states[this.index][2];
			this.color.selectedIndex  = this.tSwitch.states[this.index][3];

			switch (this.tSwitch.states[this.index][3])
			{
				case "darkgrey":  this.color.selectedIndex = DARKGREY; break;
				case "lightgrey": this.color.selectedIndex = LIGHTGREY; break;
				default:          this.color.selectedIndex = BLACK;
			}
		}
	};

	this.accept = function()
	{
		var data = this.generateXml();
		this.control.saveSwitches(data);
		this.control.update(this.control.UPDATE_ALL);
	};

	this.generateXml = function()
	{
		var sResult = "" +
"<?xml version=\"1.0\"?>\n" +
"<collection>\n";
		for (var i = 0; i < this.listbox.getRowCount(); i++)
		{
			var tSwitch = this.listbox.getItemAtIndex(i).sw;
			if(tSwitch == null) continue;
			sResult += "" +
"	<switch name=\"" + tSwitch.name + "\"\n" +
"		show=\"" + tSwitch.show + "\"\n" +
"		preference=\"" + tSwitch.preference + "\"\n" +
"		initial=\"" + tSwitch.initial + "\"\n" +
"		type=\"" + tSwitch.type + "\"\n" +
"		tooltipPrefix=\"" + tSwitch.tooltipPrefix + "\">\n";
			for (var j = 0; j < tSwitch.states.length; j++)
			{
				sResult += "" +
"		<state  value=\"" + tSwitch.states[j][0] + "\"\n" +
"			description=\"" + tSwitch.states[j][1] + "\"\n" +
"			tooltipPostfix=\"" + tSwitch.states[j][2] + "\"\n" +
"			color=\"" + tSwitch.states[j][3] + "\" />\n";
			}
			sResult += "" +
"	</switch>\n";
		}
		sResult += "" +
"</collection>\n";
		return sResult;
	};
}

function TreeView(table, columns, rowcount)
{
	this.table               = table;
	this.columns             = columns;
	this.rowCount            = rowcount;
	this.cycleHeader         = function(column) { sortColumn(column); };
	this.getCellProperties   = function(row, col, props) {};
	this.getCellText         = function(row, column) { return this.table[row][this.columns[column.id][0]]; };
	this.getColumnProperties = function(colid, col, props) {};
	this.getImageSrc         = function(row, col) { return null; };
	this.getLevel            = function(row)      { return 0; };
	this.getRowProperties    = function(row, props) {};
	this.isContainer         = function(row)     { return false; };
	this.isSeparator         = function(row)     { return false; };
	this.isSorted            = function(row)     { return false; };
	this.setTree             = function(treebox) { this.treebox=treebox; };
}

// -----------------------------------------------------------------------------

var switchDialog = new SwitchDialog();
